@@ -4,13 +4,12 @@ from lark.exceptions import LarkError
 import logging
 from sureberus import schema as S
 
-from .utils import coerce_format, coerce_pop_version, _chain, SCALAR_TYPES
-from .field_grammar import (
-    field_parser,
-    full_condition_parser,
-    noag_field_parser,
-    noag_full_condition_parser,
-    noag_any_condition_parser,
+from .utils import (
+    coerce_format,
+    coerce_pop_version,
+    coerce_shelf_meta,
+    _chain,
+    SCALAR_TYPES,
 )
 
 logging.captureWarnings(True)
@@ -70,7 +69,7 @@ def _stringify(value):
 
 
 def _convert_bucket_to_field(bucket, bucket_default_label, use_indices=False):
-    """Convert a bucket structure to a field structure.
+    """Convert a bucket structure to a if function expression field.
 
     This assumes that all conditions have been converted from
     partial conditions like '<5' to full conditions like 'age<5'
@@ -112,144 +111,52 @@ def _lowercase_kind(value):
 
 
 def _save_raw_config(value):
-    """Save the original config """
-    value["_config"] = deepcopy(value)
-
+    """Save the original config excluding _config and _meta """
+    config = deepcopy(value)
+    config.pop("_config", None)
+    config.pop("_neta", None)
+    value["_config"] = config
     return value
 
 
-def _build_full_condition(field, itm):
-    """
-    Test if the itm["condition"] is a partial relation expression
-
-    If so, convert it in place into a full relation expression by including the field.
-    """
-    tree = noag_any_condition_parser.parse(itm["condition"])
-    if tree.data == "partial_relation_expr":
-        itm["condition"] = "(" + field + ")" + itm["condition"]
-
-
-def _convert_partial_conditions(value):
-    """Convert all partial conditions to full conditions in buckets and quickselects."""
-    field = value.get("field")
-
-    # Convert all bucket conditions to full conditions
-    for itm in value.get("buckets", []):
-        _build_full_condition(field, itm)
-
-    # Convert all quickselects conditions to full conditions
-    for itm in value.get("quickselects", []):
-        _build_full_condition(field, itm)
-
-    return value
-
-
-def create_buckets(value):
-    """If a validated bucket exists, convert it into a field and extra order by field."""
-    buckets = value.pop("buckets", None)
-    buckets_default_label = value.pop("buckets_default_label", None)
-    if buckets:
-        # Create a bucket
-        if "extra_fields" not in value:
-            value["extra_fields"] = []
-        bucket_field = _convert_bucket_to_field(buckets, buckets_default_label)
-        bucket_order_by_field = _convert_bucket_to_field(
-            buckets, buckets_default_label, use_indices=True
-        )
-        value["field"] = bucket_field
-        value["extra_fields"].append(
-            {"name": "order_by_expression", "field": bucket_order_by_field}
-        )
-        pass
-    return value
-
-
-def ensure_aggregation(fld):
-    """Ensure that a field has an aggregation by wrapping the entire field
-    in a sum if no aggregation is supplied."""
-    try:
-        tree = field_parser.parse(fld)
-        has_agex = list(tree.find_data("agex"))
-        if has_agex:
-            return fld
-        else:
-            return "sum(" + fld + ")"
-    except LarkError:
-        # If we can't parse we will handle this in the validator
-        return fld
-
-
 def add_version(v):
     # Add version to a parsed ingredient
     v["_version"] = "2"
     return v
 
 
-# Sureberus validators that check how a field parses
-
-
-def ParseValidator(parser):
-    def validate(field, value, error):
-        try:
-            parser.parse(value)
-        except LarkError as exc:
-            error(field, f"Error parsing field: {exc}")
-
-    return validate
-
-
-validate_parses_with_agex = ParseValidator(parser=field_parser)
-validate_parses_without_agex = ParseValidator(parser=noag_field_parser)
-validate_any_condition = ParseValidator(parser=noag_any_condition_parser)
-validate_condition = ParseValidator(parser=noag_full_condition_parser)
-validate_agex_condition = ParseValidator(parser=full_condition_parser)
-
-
 # A field that may OR MAY NOT contain an aggregation.
 # It will be the transformers responsibility to add an aggregation if one is missing
-agex_field_schema = S.String(
-    required=True, validator=validate_parses_with_agex, coerce=ensure_aggregation
-)
+field_schema = S.String(required=True)
 
-# A field that is guaranteed to not contain an aggregation
-noag_field_schema = S.String(required=True, validator=validate_parses_without_agex)
-
-# A full condition guaranteed to not contain an aggregation
-condition_schema = S.String(required=True, validator=validate_condition)
-
-# A full or partial contain guaranteed to not contain an aggregation
-any_condition_schema = S.String(required=True, validator=validate_any_condition)
-
-# A full condition guaranteed to not contain an aggregation
 labeled_condition_schema = S.Dict(
-    schema={"condition": condition_schema, "label": S.String(required=True)}
+    schema={"condition": field_schema, "label": S.String(required=True)}
 )
 
 # A full condition guaranteed to not contain an aggregation
 named_condition_schema = S.Dict(
-    schema={"condition": condition_schema, "name": S.String(required=True)}
+    schema={"condition": field_schema, "name": S.String(required=True)}
 )
 
 format_schema = S.String(coerce=coerce_format, required=False)
 
+
 metric_schema = S.Dict(
     schema={
-        "field": agex_field_schema,
+        "field": field_schema,
         "format": format_schema,
         "quickselects": S.List(required=False, schema=labeled_condition_schema),
     },
-    coerce=_convert_partial_conditions,
-    coerce_post=add_version,
     allow_unknown=True,
 )
 
 dimension_schema = S.Dict(
     schema={
-        "field": noag_field_schema,
+        "field": field_schema,
         "extra_fields": S.List(
             required=False,
             schema=S.Dict(
-                schema={"field": noag_field_schema, "name": S.String(required=True)}
+                schema={"field": field_schema, "name": S.String(required=True)}
             ),
         ),
         "buckets": S.List(required=False, schema=labeled_condition_schema),
@@ -258,18 +165,13 @@ dimension_schema = S.Dict(
         "lookup": S.Dict(required=False),
         "quickselects": S.List(required=False, schema=named_condition_schema),
     },
-    coerce=_chain(move_extra_fields, _convert_partial_conditions),
-    coerce_post=_chain(create_buckets, add_version),
+    coerce=move_extra_fields,
     allow_unknown=True,
 )
 
-filter_schema = S.Dict(
-    allow_unknown=True, coerce_post=add_version, schema={"condition": condition_schema}
-)
+filter_schema = S.Dict(allow_unknown=True, schema={"condition": field_schema})
 
-having_schema = S.Dict(
-    allow_unknown=True, coerce_post=add_version, schema={"condition": condition_schema}
-)
+having_schema = S.Dict(allow_unknown=True, schema={"condition": field_schema})
 
 ingredient_schema = S.Dict(
     choose_schema=S.when_key_is(
@@ -283,12 +185,104 @@ ingredient_schema = S.Dict(
         default_choice="metric",
     ),
     coerce=_chain(_lowercase_kind, _save_raw_config),
+    coerce_post=add_version,
     registry={},
 )
 
+
+strict_metric_schema = S.Dict(
+    allow_unknown=False,
+    schema={
+        "_version": S.String(default="2"),
+        "_shelf_meta": S.Dict(required=False),
+        "icon": S.String(required=False),
+        "_meta": S.Dict(required=False),
+        "_config": S.Dict(required=False),
+        "singular": S.String(required=False),
+        "plural": S.String(required=False),
+        "field": field_schema,
+        "format": format_schema,
+        "quickselects": S.List(required=False, schema=labeled_condition_schema),
+    },
+)
+
+strict_dimension_schema = S.Dict(
+    allow_unknown=False,
+    schema={
+        "_version": S.String(default="2"),
+        "_shelf_meta": S.Dict(required=False),
+        "icon": S.String(required=False),
+        "_meta": S.Dict(required=False),
+        "_config": S.Dict(required=False),
+        "singular": S.String(required=False),
+        "plural": S.String(required=False),
+        "field": field_schema,
+        "extra_fields": S.List(
+            required=False,
+            schema=S.Dict(
+                schema={"field": field_schema, "name": S.String(required=True)}
+            ),
+        ),
+        "buckets": S.List(required=False, schema=labeled_condition_schema),
+        "buckets_default_label": {"anyof": SCALAR_TYPES, "required": False},
+        "format": format_schema,
+        "lookup": S.Dict(required=False),
+        "quickselects": S.List(required=False, schema=named_condition_schema),
+    },
+    coerce=move_extra_fields,
+)
+
+strict_filter_schema = S.Dict(
+    allow_unknown=False,
+    schema={
+        "_version": S.String(default="2"),
+        "_shelf_meta": S.Dict(required=False),
+        "condition": field_schema,
+    },
+)
+
+strict_having_schema = S.Dict(
+    allow_unknown=False,
+    schema={
+        "_version": S.String(default="2"),
+        "_shelf_meta": S.Dict(required=False),
+        "condition": field_schema,
+    },
+)
+
+
+# A schema for ingredients that only allows specified fields
+strict_ingredient_schema = S.Dict(
+    choose_schema=S.when_key_is(
+        "kind",
+        {
+            "metric": strict_metric_schema,
+            "dimension": strict_dimension_schema,
+            "filter": strict_filter_schema,
+            "having": strict_having_schema,
+        },
+        default_choice="metric",
+    ),
+    coerce=_chain(_lowercase_kind, _save_raw_config),
+    coerce_post=add_version,
+    registry={},
+)
+
+
+# A schema for a shelf with fields turned into sqlalchemy using lark parsing
+# and allow_unknown properties on the definitions
 shelf_schema = S.Dict(
     valueschema=ingredient_schema,
     keyschema=S.String(),
-    coerce=_chain(coerce_pop_version, coerce_replace_refs),
+    coerce=_chain(coerce_pop_version, coerce_shelf_meta, coerce_replace_refs),
+    allow_unknown=True,
+)
+
+# A schema for a shelf with fields turned into sqlalchemy using lark parsing
+# and NO allow_unknown on the ingredient definitions
+strict_shelf_schema = S.Dict(
+    valueschema=strict_ingredient_schema,
+    keyschema=S.String(),
+    coerce=_chain(coerce_pop_version, coerce_shelf_meta, coerce_replace_refs),
     allow_unknown=True,
 )
