@@ -5,6 +5,7 @@ import string
 import unicodedata
 from functools import wraps
 from inspect import isclass, getfullargspec
+from uuid import uuid4
 
 import attr
 import sqlalchemy.orm
@@ -16,6 +17,7 @@ from faker.providers import BaseProvider
 from sqlalchemy.engine.default import DefaultDialect
 from sqlalchemy.sql.functions import FunctionElement
 from sqlalchemy.sql.sqltypes import Date, DateTime, NullType, String
+from sqlalchemy.exc import UnsupportedCompilationError
 
 # only expose the printing sql function
 __all__ = [
@@ -30,18 +32,21 @@ __all__ = [
 
 def filter_to_string(filt):
     """Compile a filter object to a literal string"""
-    if hasattr(filt, "filters") and filt.filters:
-        return str(filt.filters[0].compile(compile_kwargs={"literal_binds": True}))
-    elif hasattr(filt, "havings") and filt.havings:
-        return str(filt.havings[0].compile(compile_kwargs={"literal_binds": True}))
-    elif isinstance(filt, bool):
-        return str(filt)
-    else:
-        return str(filt.compile(compile_kwargs={"literal_binds": True}))
+    try:
+        if hasattr(filt, "filters") and filt.filters:
+            return str(filt.filters[0].compile(compile_kwargs={"literal_binds": True}))
+        elif hasattr(filt, "havings") and filt.havings:
+            return str(filt.havings[0].compile(compile_kwargs={"literal_binds": True}))
+        elif isinstance(filt, bool):
+            return str(filt)
+        else:
+            return str(filt.compile(compile_kwargs={"literal_binds": True}))
+    except UnsupportedCompilationError:
+        return uuid4()
 
 
 def generate_faker_seed(value):
-    """Generate a seed value for faker. """
+    """Generate a seed value for faker."""
     if not isinstance(value, str):
         value = str(value)
 
@@ -90,7 +95,7 @@ class TestProvider(BaseProvider):
 
 
 class StringLiteral(String):
-    """ Teach SA how to literalize various things. """
+    """Teach SA how to literalize various things."""
 
     def literal_processor(self, dialect):
         super_processor = super(StringLiteral, self).literal_processor(dialect)
@@ -149,7 +154,7 @@ WHITESPACE_RE = re.compile(r"\s+", flags=re.DOTALL | re.MULTILINE)
 
 
 def replace_whitespace_with_space(s):
-    """ Replace multiple whitespaces with a single space. """
+    """Replace multiple whitespaces with a single space."""
     return WHITESPACE_RE.sub(" ", s)
 
 
